/*
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 */

#ifndef RAST_DB_H
#define RAST_DB_H

/**
 * @file db.h db
 */

#include <apr_pools.h>
#include <apr_file_io.h>

#include "rast/rast.h"
#include "rast/error.h"
#include "rast/encoding.h"
#include "rast/text_index.h"
#include "rast/property.h"

/**
 * @defgroup db db
 * @{
 */

/** Options for rast_db_create() */
typedef struct {
    /** The byte order of the database */
    rast_byte_order_e byte_order;
    /** The encoding of the database */
    const char *encoding;
    /** Whether the whole text should be preseved in the database */
    int preserve_text;
    /** The property definitions */
    rast_property_t *properties;
    /** The number of the properties */
    rast_size_t num_properties;
    /** The block size of .pos files */
    rast_size_t pos_block_size;
} rast_db_create_option_t;

/** Options for rast_db_open() */
typedef struct {
    /** The number of registered characters to start sync */
    int sync_threshold_chars; 
} rast_db_open_option_t;

/** A structure that represents an index to search properties */
typedef struct {
    /** The index to search properties */
    DB *inv;
    /** The index to search text in properties */
    rast_text_index_t *text;
} rast_property_index_t;

/** A structure that represents a database to search documents */
typedef struct {
    /** The pool associated with the database */
    apr_pool_t *pool;
    /** The path of the database */
    const char *path;
    /**
     * @param flags for rast_db_open()
     *          - RAST_RDRW
     *            - open for reading and writing
     *          - RAST_RDONLY
     *            - open for reading
     */
    int flags;
    /** The encoding of the database */
    const char *encoding;
    /** The encoding module of the database */
    rast_encoding_module_t *encoding_module;
    /** Whether the whole text should be preseved in the database */
    int preserve_text;
    /** Whether the numbers in the database are in the natvie byte order */
    int is_native; 
    /** The property definitions */
    rast_property_t *properties;
    /** The number of properties */
    rast_size_t num_properties;
    /** The block size of .pos files */
    rast_size_t pos_block_size;
    /** The index to search text */
    rast_text_index_t *text_index;
    /** The database that holds properties */
    DB *properties_db;
    /** The indices to search properties */
    rast_property_index_t *property_indices;
    /** The Berkeley DB environment */
    DB_ENV *bdb_env;
    /** The Berkeley DB transaction */
    DB_TXN *bdb_txn;
    /** The lock file */
    apr_file_t *lock_file;
    /** The number of registered characters to start sync */
    int sync_threshold_chars;
    /** The number of registered characters */
    int registered_chars;
} rast_db_t;

/**
 * Create options for rast_db_create().
 * @param pool The pool to allocate the memory out of
 * @return The created options
 */
rast_db_create_option_t *rast_db_create_option_create(apr_pool_t *pool);

/**
 * Create a database.  This function do not open the created database.
 * Use rast_db_open() to open the database.
 * @param name The name of the database directory
 * @param options The options to create a database.
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_db_create_option_create()
 */
rast_error_t *rast_db_create(const char *name,
                             rast_db_create_option_t *options,
                             apr_pool_t *pool);

/**
 * Create options for rast_db_open().
 * @param pool The pool to allocate the memory out of
 * @return The created options
 */
rast_db_open_option_t *rast_db_open_option_create(apr_pool_t *pool);

/**
 * Open the specified database.
 * @param db The database opened
 * @param name The name of the database
 * @param flags How to open
 *          - RAST_RDRW
 *            - open for reading and writing
 *          - RAST_RDONLY
 *            - open for reading
 * @param options The options to open the database.
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_db_open_option_create()
 */
rast_error_t *rast_db_open(rast_db_t **db, const char *name, int flags,
                           rast_db_open_option_t *options, apr_pool_t *pool);

/**
 * Sync registered data to files.
 * @param db The database to sync
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_sync(rast_db_t *db);

/**
 * Close the database.
 * @param db The database to close
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_close(rast_db_t *db);

/**
 * Return the byte order of the database.
 * @param db The database
 * @return The byte order of the database
 */
rast_byte_order_e rast_db_byte_order(rast_db_t *db);

/**
 * Return the encoding of the database.
 * @param db The database
 * @return The encoding of the database
 */
const char *rast_db_encoding(rast_db_t *db);

/**
 * Return the property definitions of the database.
 * @param db The database
 * @param num_properties The number of properties
 * @return The property definitions
 */
const rast_property_t *rast_db_properties(rast_db_t *db,
                                          rast_size_t *num_properties);

/**
 * Return the number of registered characters to start sync.
 * @param db The database
 * @return The number of registered characters to start sync
 */
int rast_db_sync_threshold_chars(rast_db_t *db);

/**
 * Register a document to the database.
 * @param db The database
 * @param text The text of the document
 * @param nbytes The length of text in bytes
 * @param properties The property values of documents
 * @param doc_id If non-NULL, the registered document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_register(rast_db_t *db,
                               const char *text, rast_size_t nbytes,
                               rast_value_t *properties,
                               rast_doc_id_t *doc_id);

/**
 * Search documents in the database.
 * @param db The database
 * @param query The query string
 * @param options search options
 * @param result The search result
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_search_option_create()
 */
rast_error_t *rast_db_search(rast_db_t *db, const char *query,
                             rast_search_option_t *options,
                             rast_result_t **result, apr_pool_t *pool);

/**
 * Delete the document from the database.
 * @param db The database
 * @param doc_id The document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_delete(rast_db_t *db, rast_doc_id_t doc_id);

/**
 * Update the document in the database.
 * @param db The database
 * @param old_doc_id The document ID
 * @param text The text of the document
 * @param nbytes The length of text in bytes
 * @param properties The property values of documents
 * @param new_doc_id If non-NULL, Updated document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_update(rast_db_t *db, rast_doc_id_t old_doc_id,
                             const char *text, rast_size_t nbytes,
                             rast_value_t *properties,
                             rast_doc_id_t *new_doc_id);

/**
 * Get the text of the document from the database.
 * @param db The database
 * @param doc_id The document ID
 * @param s The text of the document
 * @param nbytes The length of the text in bytes
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_get_text(rast_db_t *db,
                               rast_doc_id_t doc_id,
                               char **s, rast_size_t *nbytes,
                               apr_pool_t *pool);

/**
 * Compare keys in the property index.
 * @param type The type of the property
 * @param db The Berkeley DB handle
 * @param x The left key to compare
 * @param y The right key to compare
 * @return 0 if x == y, positive value if x > y, negative value otherwise.
 */
int rast_compare_keys(rast_type_e type, DB *db, const DBT *x, const DBT *y);

/** @} */

#endif

/* vim: set filetype=c sw=4 expandtab : */
